/*
 * n01_stats
 *
 * file.c
 *
 * Copyright (C) 1996-2006 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <shlobj.h>

#include "general.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "file.h"

/* Define */
#define BUF_SIZE				256

/* Global Variables */

/* Local Function Prototypes */

/*
 * file_find - t@Č
 */
BOOL file_find(const TCHAR *path, const TCHAR *filename, TCHAR *ret)
{
	WIN32_FIND_DATA find_data;
	HANDLE h_find_file;
	TCHAR find_path[MAX_PATH];
	TCHAR *p;
	BOOL rc = FALSE;

	// t@C
	wsprintf(find_path, TEXT("%s\\%s"), path, filename);
	if ((h_find_file = FindFirstFile(find_path, &find_data)) != INVALID_HANDLE_VALUE &&
		!(find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
		if (GetFullPathName(find_path, MAX_PATH - 1, ret, &p) == 0) {
			lstrcpy(ret, find_path);
		}
		FindClose(h_find_file);
		return TRUE;
	}
	FindClose(h_find_file);

	// ̊Kw
	wsprintf(find_path, TEXT("%s\\*"), path);
	if ((h_find_file = FindFirstFile(find_path, &find_data)) == INVALID_HANDLE_VALUE) {
		return FALSE;
	}
	do {
		if (find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY &&
			lstrcmp(find_data.cFileName, TEXT(".")) != 0 && lstrcmp(find_data.cFileName, TEXT("..")) != 0) {
			wsprintf(find_path, TEXT("%s\\%s"), path, find_data.cFileName);
			if (file_find(find_path, filename, ret) == TRUE) {
				rc = TRUE;
				break;
			}
		}
	} while (FindNextFile(h_find_file, &find_data) == TRUE);
	FindClose(h_find_file);
	return rc;
}

/*
 * file_find_list - t@CăpX
 */
BOOL file_find_list(const TCHAR *path, FILE_FIND_INFO *ffi, const int cnt)
{
	WIN32_FIND_DATA find_data;
	HANDLE h_find_file;
	TCHAR find_path[MAX_PATH];
	TCHAR file_name[FILE_NAME_SIZE];
	TCHAR *p;
	UINT hash;
	int i;
	BOOL rc = FALSE;

	wsprintf(find_path, TEXT("%s\\*"), path);
	if ((h_find_file = FindFirstFile(find_path, &find_data)) == INVALID_HANDLE_VALUE) {
		return rc;
	}
	do {
		if (find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			if (lstrcmp(find_data.cFileName, TEXT(".")) != 0 && lstrcmp(find_data.cFileName, TEXT("..")) != 0) {
				// ̊Kw
				wsprintf(find_path, TEXT("%s\\%s"), path, find_data.cFileName);
				if (file_find_list(find_path, ffi, cnt) == TRUE) {
					rc = TRUE;
					break;
				}
			}
			continue;
		}
		hash = str2hash(find_data.cFileName);
		lstrcpyn(file_name, find_data.cFileName, FILE_NAME_SIZE - 1);
		for (i = 0; i < cnt; i++) {
			if (ffi[i].hash == hash && lstrcmpi(ffi[i].name, file_name) == 0) {
				ffi[i].hash = 0;
				// pXݒ
				wsprintf(find_path, TEXT("%s\\%s"), path, find_data.cFileName);
				if (GetFullPathName(find_path, MAX_PATH - 1, ffi[i].name, &p) == 0) {
					lstrcpy(ffi[i].name, find_path);
				}
				break;
			}
		}
	} while (FindNextFile(h_find_file, &find_data) == TRUE);
	FindClose(h_find_file);
	return rc;
}

/*
 * file_get_size - t@C̃TCY擾
 */
DWORD file_get_size(const TCHAR *file_name)
{
	WIN32_FIND_DATA FindData;
	HANDLE hFindFile;

	if ((hFindFile = FindFirstFile(file_name, &FindData)) == INVALID_HANDLE_VALUE) {
		return -1;
	}
	FindClose(hFindFile);

	if ((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0) {
		// fBNgł͂Ȃꍇ̓TCYԂ
		return FindData.nFileSizeLow;
	}
	return 0;
}

/*
 * file_check_file - t@C݂邩`FbN
 */
BOOL file_check_file(const TCHAR *path)
{
	WIN32_FIND_DATA FindData;
	HANDLE hFindFile;

	if ((hFindFile = FindFirstFile(path, &FindData)) == INVALID_HANDLE_VALUE) {
		return FALSE;
	}
	FindClose(hFindFile);

	if ((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0) {
		// t@C݂ꍇ
		return TRUE;
	}
	return FALSE;
}

/*
 * file_check_directory - fBNg݂邩`FbN
 */
BOOL file_check_directory(const TCHAR *path)
{
	WIN32_FIND_DATA FindData;
	HANDLE hFindFile;

	if ((hFindFile = FindFirstFile(path, &FindData)) == INVALID_HANDLE_VALUE) {
		return FALSE;
	}
	FindClose(hFindFile);

	if (FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
		//fBNg݂ꍇ
		return TRUE;
	}
	return FALSE;
}

/*
 * create_tree_directory - fBNgKw쐬
 */
void create_tree_directory(const TCHAR *p)
{
	TCHAR tmp[BUF_SIZE];
	TCHAR *r;

	r = tmp;
	while (*p != TEXT('\0')) {
#ifndef UNICODE
		if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
			*(r++) = *(p++);
			*(r++) = *(p++);
		} else
#endif
		if (*p == TEXT('\\') || *p == TEXT('/')) {
			*r = TEXT('\0');
			//fBNg̍쐬
			if (file_check_directory(tmp) == FALSE) {
				CreateDirectory(tmp, NULL);
			}
			*(r++) = *(p++);
		} else {
			*(r++) = *(p++);
		}
	}
	*r = TEXT('\0');
	if (file_check_directory(tmp) == FALSE) {
		CreateDirectory(tmp, NULL);
	}
}

/*
 * select_folder - tH_I
 */
BOOL select_folder(const HWND hWnd, TCHAR *title, TCHAR *ret)
{
	BROWSEINFO bi;
	LPITEMIDLIST lpItemList;
	TCHAR buf[BUF_SIZE];

	bi.hwndOwner = hWnd;
	bi.pidlRoot = (const struct _ITEMIDLIST *)CSIDL_DESKTOP;
	*buf = TEXT('\0');
	bi.pszDisplayName = buf;
	bi.lpszTitle = title;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = NULL;
	bi.lParam = 0;
	bi.iImage = 0;
	// tH_Iʂ\
	lpItemList = SHBrowseForFolder(&bi);
	if(lpItemList == NULL){
		return FALSE;
	}
	// IĂtH_̃tpX擾
	SHGetPathFromIDList(lpItemList, ret);
	return TRUE;
}

/*
 * file_read_buf - t@Cǂݍ
 */
BYTE *file_read_buf(const TCHAR *path, DWORD *ret_size, TCHAR *err_str)
{
	HANDLE hFile;
	DWORD size;
	DWORD ret;
	BYTE *buf;

	// t@CJ
	hFile = CreateFile(path, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == NULL || hFile == (HANDLE)-1) {
		message_get_error(GetLastError(), err_str);
		return NULL;
	}
	if ((size = GetFileSize(hFile, NULL)) == 0xFFFFFFFF) {
		message_get_error(GetLastError(), err_str);
		CloseHandle(hFile);
		return NULL;
	}

	if ((buf = (BYTE *)mem_alloc(size + 1)) == NULL) {
		message_get_error(GetLastError(), err_str);
		CloseHandle(hFile);
		return NULL;
	}
	// t@Cǂ݂
	if (ReadFile(hFile, buf, size, &ret, NULL) == FALSE) {
		message_get_error(GetLastError(), err_str);
		mem_free(&buf);
		CloseHandle(hFile);
		return NULL;
	}
	CloseHandle(hFile);

	if (ret_size != NULL) {
		*ret_size = size;
	}
	return buf;
}

/*
 * file_write_buf - t@Cɏ
 */
BOOL file_write_buf(const TCHAR *path, const BYTE *data, const DWORD size, TCHAR *err_str)
{
	HANDLE hFile;
	DWORD ret;

	// t@CJ
	hFile = CreateFile(path, GENERIC_READ | GENERIC_WRITE, 0, 0, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == NULL || hFile == (HANDLE)-1) {
		message_get_error(GetLastError(), err_str);
		return FALSE;
	}
	// t@C̏
	if (WriteFile(hFile, data, size, &ret, NULL) == FALSE) {
		message_get_error(GetLastError(), err_str);
		CloseHandle(hFile);
		return FALSE;
	}
	FlushFileBuffers(hFile);
	CloseHandle(hFile);
	return TRUE;
}
/* End of source */
