/*
 * n01_stats
 *
 * main.c
 *
 * Copyright (C) 1996-2006 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <shlobj.h>
#include <commctrl.h>

#include "general.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "ini.h"
#include "file.h"
#include "csv.h"
#include "data.h"
#include "index.h"
#include "ImageList.h"
#include "TreeView.h"
#include "ListView.h"
#include "ToolBar.h"
#include "StatusBar.h"
#include "Frame.h"
#include "filter.h"
#include "column.h"
#include "view.h"
#include "player.h"
#include "score_list.h"
#include "delete.h"
#include "split.h"
#include "option.h"

#include "resource.h"

/* Define */
#define MAIN_WND_CLASS				TEXT("n01_stats_wnd")
#define WINDOW_TITLE				TEXT("n01_stats")

#define MUTEX						TEXT("_n01_stats_Mutex_")
#define INI_FILE					TEXT("n01_stats.ini")
#define INDEX_FILE					TEXT("n01_stats.idx")
#define CSV_PATH					TEXT("data")
#define OUT_DATA_PATH				TEXT("output_data")
#define OUT_LIST_PATH				TEXT("output_list")

#define ID_TREE						100
#define ID_LIST						101
#define ID_TOOLBAR					102
#define ID_STATUSBAR				103

// ^C}[ID
#define ID_TIMER_INIT				1
#define ID_TIMER_SEP				2

#define MENU_POS_VIEW				1
#define MENU_POS_DATA				2

/* Global Variables */
HINSTANCE hInst;
static HANDLE hAccel;
HWND g_main_wnd;
static TCHAR work_path[MAX_PATH];
static TCHAR ini_file[MAX_PATH];
TCHAR index_file[MAX_PATH];
TCHAR csv_path[MAX_PATH];
TCHAR out_data_path[MAX_PATH];
TCHAR out_list_path[MAX_PATH];
static int g_st;
static int g_file_all;

OPTION op;

DATA_INFO *g_di;
int g_di_count;
int g_di_size;

DATA_INFO *g_show_di;
int g_show_di_count;
int g_show_di_size;

EXTRACT_INFO exi;

NAME_INFO *nmi;
int nmi_count;
int nmi_size;
static SYSTEMTIME *dti;
static int dti_count;
static int dti_size;

/* Local Function Prototypes */
static void server_start(const HWND hWnd);
static BOOL treeinfo_create_list(const DATA_INFO *di, const int st, const int en);
static void treeinfo_free(void);
static int data_to_hdrop_name_size(const HWND hWnd);
static FILE_FIND_INFO *data_to_hdrop_name(const HWND hWnd, int *cnt);
static HDROP data_to_hdrop(const HWND hWnd);
static BOOL data_to_clipboard(const HWND hWnd);
static BOOL hdrop_to_file(const HWND hWnd, const HDROP hdrop);
static void data_show_check(const HWND hWnd);
static void data_delete(const HWND hWnd);
static void sync_treeview_node(const HWND hWnd, HTREEITEM root_item);
static void set_treeview_node(const HWND hWnd, HTREEITEM root_item);
static void set_treeview_root(const HWND hWnd);
static void set_toolbar(const HWND hWnd, UINT id, int check);
static void set_statusbar_count(const HWND hWnd);
static void set_controls(const HWND hWnd);
static void set_menu(const HWND hWnd);
static LRESULT CALLBACK main_proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
static void get_path(const HINSTANCE hInstance);
static BOOL init_application(HINSTANCE hInstance);
static HWND init_instance(HINSTANCE hInstance, int CmdShow);

/*
 * server_start - T[oJn
 */
static void server_start(const HWND hWnd)
{
	HINSTANCE sock_lib;
	FARPROC sock_server_start;

	if ((sock_lib = LoadLibrary(TEXT("n01_sock.n01"))) == NULL) {
		return;
	}
	sock_server_start = GetProcAddress(sock_lib, "server_start");
	if (sock_server_start != NULL) {
		sock_server_start(hWnd);
	}
}

/*
 * treeinfo_create_list - c[̍쐬
 */
static BOOL treeinfo_create_list(const DATA_INFO *di, const int st, const int en)
{
	NAME_INFO *tmp_nmi;
	SYSTEMTIME *tmp_dti;
	int i, j;

	for (i = st; i < en; i++) {
		if (di[i].status == DATA_STATUS_DELETE) {
			// 폜ς
			continue;
		}
		// OXg̐
		for (j = 0; j < nmi_count; j++) {
			if (di[i].name_hash == nmi[j].name_hash && lstrcmpi(di[i].name, nmi[j].name) == 0) {
				break;
			}
		}
		if (j >= nmi_count) {
			if (nmi_count + 1 >= nmi_size) {
				nmi_size += DATA_ALLOC_SIZE;
				tmp_nmi = mem_alloc(sizeof(NAME_INFO) * nmi_size);
				if (tmp_nmi == NULL) {
					return FALSE;
				}
				if (nmi != NULL) {
					CopyMemory(tmp_nmi, nmi, sizeof(NAME_INFO) * nmi_count);
					mem_free(&nmi);
				}
				nmi = tmp_nmi;
			}
			lstrcpy(nmi[nmi_count].name, di[i].name);
			nmi[nmi_count].name_hash = di[i].name_hash;
			nmi_count++;
		}
		// tXg̐
		for (j = 0; j < dti_count; j++) {
			if (di[i].start_time.wYear == dti[j].wYear &&
				di[i].start_time.wMonth == dti[j].wMonth &&
				di[i].start_time.wDay == dti[j].wDay) {
				break;
			}
		}
		if (j >= dti_count) {
			if (dti_count + 1 >= dti_size) {
				dti_size += DATA_ALLOC_SIZE;
				tmp_dti = mem_alloc(sizeof(SYSTEMTIME) * dti_size);
				if (tmp_dti == NULL) {
					return FALSE;
				}
				if (dti != NULL) {
					CopyMemory(tmp_dti, dti, sizeof(SYSTEMTIME) * dti_count);
					mem_free(&dti);
				}
				dti = tmp_dti;
			}
			dti[dti_count] = di[i].start_time;
			dti_count++;
		}
	}
	return TRUE;
}

/*
 * treeinfo_free - c[̉
 */
static void treeinfo_free(void)
{
	mem_free(&nmi);
	nmi_count = 0;
	nmi_size = 0;
	mem_free(&dti);
	dti_count = 0;
	dti_size = 0;
}

/*
 * data_to_hdrop_name_size - hbvt@C̃TCY擾
 */
static int data_to_hdrop_name_size(const HWND hWnd)
{
	DATA_INFO *di;
	int i, j;
	int ret;

	ret = 0;
	i = -1;
	while ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), i, LVNI_SELECTED)) != -1) {
		di = &g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)];
		if (di->index >= 0) {
			ret++;
		} else {
			// f[^
			for (j = 0; j < g_di_count; j++) {
				if (data_extract_check(&g_di[j]) == FALSE ||
					data_marge_check(di, &g_di[j]) == FALSE) {
					continue;
				}
				ret++;
			}
		}
	}
	return ret;
}

/*
 * data_to_hdrop_name - hbvt@C̍쐬
 */
static FILE_FIND_INFO *data_to_hdrop_name(const HWND hWnd, int *cnt)
{
	DATA_INFO *di;
	FILE_FIND_INFO *ffi;
	int i, j;

	// TCY擾
	i = data_to_hdrop_name_size(hWnd);
	ffi = mem_alloc(sizeof(FILE_FIND_INFO) * i);
	if (ffi == NULL) {
		return NULL;
	}
	*cnt = 0;

	i = -1;
	while ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), i, LVNI_SELECTED)) != -1) {
		di = &g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)];
		if (di->index >= 0) {
			lstrcpy(ffi[*cnt].name, g_di[di->index].file_name);
			ffi[*cnt].hash = g_di[di->index].file_name_hash;
			(*cnt)++;
		} else {
			// f[^
			for (j = 0; j < g_di_count; j++) {
				if (data_extract_check(&g_di[j]) == FALSE ||
					data_marge_check(di, &g_di[j]) == FALSE) {
					continue;
				}
				lstrcpy(ffi[*cnt].name, g_di[j].file_name);
				ffi[*cnt].hash = g_di[j].file_name_hash;
				(*cnt)++;
			}
		}
	}
	return ffi;
}

/*
 * data_to_hdrop - hbvt@C̍쐬
 */
static HDROP data_to_hdrop(const HWND hWnd)
{
#ifndef UNICODE
	OSVERSIONINFO os_info;
#endif
	HDROP hDrop;
	LPDROPFILES lpDropFile;
	TCHAR *buf;
	BOOL fWide = FALSE;
	int flen = 0;
	int i;
	FILE_FIND_INFO *ffi;
	int ffi_count;

#ifdef UNICODE
	fWide = TRUE;
#else
	// OS̃o[W擾
	os_info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&os_info);
	if (os_info.dwPlatformId == VER_PLATFORM_WIN32_NT) {
		fWide = TRUE;
	}
#endif
	// t@C̃Xg쐬
	if ((ffi = data_to_hdrop_name(hWnd, &ffi_count)) == NULL) {
		return NULL;
	}
	// pX̌
	file_find_list(csv_path, ffi, ffi_count);
	// TCYvZ
#ifdef UNICODE
	for (i = 0; i < ffi_count; i++) {
		if (ffi[i].hash == 0) {
			flen += (lstrlen(ffi[i].name) + 1) * sizeof(TCHAR);
		}
	}
	flen += sizeof(TCHAR);
#else
	if (fWide == TRUE) {
		// ChL
		for (i = 0; i < ffi_count; i++) {
			if (ffi[i].hash == 0) {
				flen += MultiByteToWideChar(CP_ACP, 0, ffi[i].name, -1, NULL, 0) * sizeof(WCHAR);
			}
		}
		flen += sizeof(WCHAR);
	} else {
		// }`oCg
		for (i = 0; i < ffi_count; i++) {
			if (ffi[i].hash == 0) {
				flen += lstrlen(ffi[i].name) + 1;
			}
		}
		flen++;
	}
#endif
	if ((hDrop = (HDROP)GlobalAlloc(GHND, sizeof(DROPFILES) + flen)) == NULL) {
		mem_free(&ffi);
		return NULL;
	}
	lpDropFile = (LPDROPFILES)GlobalLock(hDrop);
	lpDropFile->pFiles = sizeof(DROPFILES);
	lpDropFile->pt.x = 0;
	lpDropFile->pt.y = 0;
	lpDropFile->fNC = FALSE;
	lpDropFile->fWide = fWide;
#ifdef UNICODE
	buf = (TCHAR *)(lpDropFile + 1);
	for (i = 0; i < ffi_count; i++) {
		if (ffi[i].hash == 0) {
			lstrcpy(buf, ffi[i].name);
			buf += lstrlen(buf) + 1;
		}
	}
#else
	// \̂̌Ƀt@C̃XgRs[(t@C\0t@C\0t@C\0\0)
	if (fWide == TRUE) {
		// ChL
		WCHAR *buf;
		int len;

		buf = (WCHAR *)(lpDropFile + 1);
		for (i = 0; i < ffi_count; i++) {
			if (ffi[i].hash == 0) {
				len = MultiByteToWideChar(CP_ACP, 0, ffi[i].name, -1, buf, MAX_PATH);
				buf += len;
			}
		}
	} else {
		// }`oCg
		buf = (char *)(lpDropFile + 1);
		for (i = 0; i < ffi_count; i++) {
			if (ffi[i].hash == 0) {
				lstrcpy(buf, ffi[i].name);
				buf += lstrlen(buf) + 1;
			}
		}
	}
#endif
	mem_free(&ffi);
	GlobalUnlock(hDrop);
	return hDrop;
}

/*
 * data_to_clipboard - f[^Nbv{[hɐݒ
 */
static BOOL data_to_clipboard(const HWND hWnd)
{
	HCURSOR old_cursor;
	HANDLE hMem;
	HDROP hdrop = NULL;
	UINT fmt;

	old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	// eLXg`̍쐬
	hMem = csv_copy_list(GetDlgItem(hWnd, ID_LIST), g_show_di, g_show_di_count);
	// t@C`̍쐬
	hdrop = data_to_hdrop(hWnd);
	// Nbv{[hJ
	if (OpenClipboard(hWnd) == FALSE) {
		SetCursor(old_cursor);
		return FALSE;
	}
	if (EmptyClipboard() == FALSE) {
		CloseClipboard();
		SetCursor(old_cursor);
		return FALSE;
	}
	// eLXg`Nbv{[hɐݒ
	if (hMem != NULL) {
#ifdef UNICODE
		SetClipboardData(CF_UNICODETEXT, hMem);
#else
		SetClipboardData(CF_TEXT, hMem);
#endif
	}
	// CSV`Nbv{[hɐݒ
	if (hMem != NULL && (fmt = RegisterClipboardFormat(TEXT("Csv"))) != 0) {
		SetClipboardData(fmt, hMem);
	}
	// t@C`Nbv{[hɐݒ
	if (hdrop != NULL) {
		SetClipboardData(CF_HDROP, hdrop);
	}
	CloseClipboard();
	SetCursor(old_cursor);
	return TRUE;
}

/*
 * hdrop_to_file - hbvt@C̓eŃt@CRs[
 */
static BOOL hdrop_to_file(const HWND hWnd, const HDROP hdrop)
{
	HCURSOR old_cursor;
	SHFILEOPSTRUCT shfo;
	TCHAR *buf;
	TCHAR tmp[MAX_PATH];
	TCHAR *p, *r;
	BOOL ret;
	int len;
	int cnt;
	int i;

	// CSVt@Ĉ݂̈ꗗ쐬
	len = 0;
	cnt = DragQueryFile(hdrop, 0xFFFFFFFF, NULL, 0);
	for (i = 0; i < cnt; i++) {
		len += DragQueryFile(hdrop, i, NULL, 0) + 1;
	}
	p = buf = mem_calloc(sizeof(TCHAR) * (len + 1));
	if (buf == NULL) {
		DragFinish(hdrop);
		return FALSE;
	}
	for (i = 0; i < cnt; i++) {
		len = DragQueryFile(hdrop, i, tmp, MAX_PATH - 1);
		if (str_match(TEXT("*.csv"), tmp) == TRUE) {
			lstrcpy(p, tmp);
			p += (len + 1);
		}
	}
	DragFinish(hdrop);
	if (*buf == TEXT('\0')) {
		mem_free(&buf);
		return FALSE;
	}
	// t@C̃Rs[
	shfo.hwnd = hWnd;
	shfo.wFunc = FO_COPY;
	shfo.pFrom = buf;
	shfo.pTo = csv_path;
	shfo.fFlags = 0;
	shfo.fAnyOperationsAborted = FALSE;
	shfo.hNameMappings = NULL;
	shfo.lpszProgressTitle = NULL;
	ret = SHFileOperation(&shfo);
	// \XV
	old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	p = buf;
	while (1) {
		for (r = p; *p != TEXT('\0'); p++) {
#ifndef UNICODE
			if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
				p++;
				continue;
			}
#endif	// UNICODE
			if (*p == TEXT('\\') || *p == TEXT('/')) {
				r = p + 1;
			}
		}
		SendMessage(hWnd, WM_MAIN_RELOAD_FILE, 1, (LPARAM)r);
		if (*p == TEXT('\0') && *(p + 1) == TEXT('\0')) {
			break;
		}
		p++;
	}
	mem_free(&buf);
	SendMessage(hWnd, WM_MAIN_REFRESH, 0, 0);
	SetCursor(old_cursor);
	return ret;
}

/*
 * data_show_check - \ΏۈȊÕf[^폜
 */
static void data_show_check(const HWND hWnd)
{
	DATA_INFO *di;
	int i, j;

	i = 0;
	while (i < ListView_GetItemCount(GetDlgItem(hWnd, ID_LIST))) {
		di = &g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)];
		if (di->index >= 0) {
			if (g_di[di->index].status == DATA_STATUS_DELETE) {
				di->status = DATA_STATUS_DELETE;
				ListView_DeleteItem(GetDlgItem(hWnd, ID_LIST), i);
				i--;
			}
		} else {
			for (j = 0; j < g_di_count; j++) {
				if (data_extract_check(&g_di[j]) == FALSE ||
					data_marge_check(di, &g_di[j]) == FALSE) {
					continue;
				}
				break;
			}
			if (j >= g_di_count) {
				di->status = DATA_STATUS_DELETE;
				ListView_DeleteItem(GetDlgItem(hWnd, ID_LIST), i);
				i--;
			}
		}
		i++;
	}
}

/*
 * data_delete - f[^̍폜
 */
static void data_delete(const HWND hWnd)
{
	HCURSOR old_cursor;
	DATA_INFO *di;
	TCHAR path[MAX_PATH];
	int sel;
	int i, j;
	BOOL vs_delete = FALSE;

	if ((sel = ListView_GetSelectedCount(GetDlgItem(hWnd, ID_LIST))) == 0) {
		return;
	}
	// 폜mFʂ̕\
	if (show_delete(hInst, hWnd, sel) == FALSE) {
		return;
	}
	old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)FALSE, 0);
	while ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), -1, LVNI_SELECTED)) != -1) {
		di = &g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)];
		// 폜tO̐ݒ
		di->status = DATA_STATUS_DELETE;
		if (di->index >= 0) {
			// 1폜
			g_di[di->index].status = DATA_STATUS_DELETE;
			index_save(index_file, g_di, di->index, di->index + 1);
			if (op.delete_type != 0) {
				// ΐ푊폜
				g_di[g_di[di->index].vs_index].status = DATA_STATUS_DELETE;
				index_save(index_file, g_di, g_di[di->index].vs_index, g_di[di->index].vs_index + 1);
				vs_delete = TRUE;
				if (op.delete_type == 2 && g_di[di->index].index == -1) {
					// CSV폜
					if (file_find(csv_path, g_di[di->index].file_name, path) == TRUE) {
						DeleteFile(path);
					}
				}
			}
		} else {
			// 폜Ώۂ̃f[^
			for (j = 0; j < g_di_count; j++) {
				if (data_extract_check(&g_di[j]) == FALSE ||
					data_marge_check(di, &g_di[j]) == FALSE) {
					continue;
				}
				// 폜tO̐ݒ
				g_di[j].status = DATA_STATUS_DELETE;
				index_save(index_file, g_di, j, j + 1);
				if (op.delete_type != 0) {
					// ΐ푊폜
					g_di[g_di[j].vs_index].status = DATA_STATUS_DELETE;
					index_save(index_file, g_di, g_di[j].vs_index, g_di[j].vs_index + 1);
					vs_delete = TRUE;
					if (op.delete_type == 2 && g_di[j].index == -1) {
						// CSV폜
						if (file_find(csv_path, g_di[j].file_name, path) == TRUE) {
							DeleteFile(path);
						}
					}
				}
			}
		}
		// Xgr[ACe폜
		ListView_DeleteItem(GetDlgItem(hWnd, ID_LIST), i);
	}
	if (vs_delete == TRUE) {
		data_show_check(hWnd);
	}
	SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)TRUE, 0);
	UpdateWindow(GetDlgItem(hWnd, ID_LIST));
	treeinfo_free();
	treeinfo_create_list(g_di, 0, g_di_count);
	sync_treeview_node(hWnd, NULL);
	SetCursor(old_cursor);
}

/*
 * data_split - f[^̕
 */
static void data_split(const HWND hWnd)
{
	HCURSOR old_cursor;
	int index;
	int sel;
	int i, j;
	BOOL delete_flag;

	if ((sel = ListView_GetSelectedCount(GetDlgItem(hWnd, ID_LIST))) == 0) {
		return;
	}
	// ݒʂ̕\
	if (show_split(hInst, hWnd) == FALSE) {
		return;
	}
	old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)FALSE, 0);
	i = -1;
	while ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), i, LVNI_SELECTED)) != -1) {
		index = listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i);
		if (g_show_di[index].index >= 0) {
			// 1
			data_split_data(hWnd, op.split_sep, op.split_delete, &g_di, &g_di_count, &g_di_size, g_show_di[index].index);
			if (g_di[g_show_di[index].index].status == DATA_STATUS_DELETE) {
				g_show_di[index].status = DATA_STATUS_DELETE;
				index_save(index_file, g_di, g_show_di[index].index, g_show_di[index].index + 1);
				ListView_DeleteItem(GetDlgItem(hWnd, ID_LIST), i);
				i--;
			}
		} else {
			// Ώۂ̃f[^
			delete_flag = FALSE;
			for (j = 0; j < g_di_count; j++) {
				if (data_extract_check(&g_di[j]) == FALSE ||
					data_marge_check(&g_show_di[index], &g_di[j]) == FALSE) {
					continue;
				}
				// 1
				data_split_data(hWnd, op.split_sep, op.split_delete, &g_di, &g_di_count, &g_di_size, j);
				if (g_di[j].status == DATA_STATUS_DELETE) {
					index_save(index_file, g_di, j, j + 1);
					delete_flag = TRUE;
				}
			}
			if (delete_flag == TRUE) {
				g_show_di[index].status = DATA_STATUS_DELETE;
				ListView_DeleteItem(GetDlgItem(hWnd, ID_LIST), i);
				i--;
			}
		}
	}
	SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)TRUE, 0);
	InvalidateRect(GetDlgItem(hWnd, ID_LIST), NULL, FALSE);
	UpdateWindow(GetDlgItem(hWnd, ID_LIST));
	treeinfo_free();
	treeinfo_create_list(g_di, 0, g_di_count);
	sync_treeview_node(hWnd, NULL);
	set_statusbar_count(hWnd);
	SetCursor(old_cursor);
}

/*
 * treeview_sync_date - tACe̓
 */
BOOL treeview_sync_date(const HWND hTreeView, const HTREEITEM hParent)
{
	HTREEITEM hItem;
	HTREEITEM delete_item;
	SYSTEMTIME date;
	int i;

	if (hParent == NULL) {
		return TRUE;
	}
	hItem = TreeView_GetChild(hTreeView, hParent);
	while (hItem != NULL) {
		if (TreeView_GetChild(hTreeView, hItem) != NULL) {
			if (treeview_sync_date(hTreeView, hItem) == FALSE) {
				return FALSE;
			}
			delete_item = hItem;
			hItem = TreeView_GetNextSibling(hTreeView, hItem);
			if (TreeView_GetChild(hTreeView, delete_item) == NULL) {
				treeview_free_node_info(hTreeView, delete_item);
				TreeView_DeleteItem(hTreeView, delete_item);
			}
		} else {
			if (treeview_get_node_date(hTreeView, hItem, &date) == FALSE) {
				return FALSE;
			}
			for (i = 0; i < dti_count; i++) {
				if (dti[i].wYear == date.wYear && dti[i].wMonth == date.wMonth && dti[i].wDay == date.wDay) {
					break;
				}
			}
			delete_item = hItem;
			hItem = TreeView_GetNextSibling(hTreeView, hItem);
			if (i >= dti_count) {
				treeview_free_node_info(hTreeView, delete_item);
				TreeView_DeleteItem(hTreeView, delete_item);
			} 
		}
	}
	return TRUE;
}

/*
 * sync_treeview_node - c[̃m[hݒ
 */
static void sync_treeview_node(const HWND hWnd, HTREEITEM root_item)
{
	HTREEITEM hItem;
	HTREEITEM delete_item;
	TCHAR name[NAME_SIZE];
	UINT hash;
	int i;

	if (root_item == NULL) {
		root_item = TreeView_GetRoot(GetDlgItem(hWnd, ID_TREE));
	}
	switch (op.extract_type) {
	case EXTRACT_TYPE_NAME:
		hItem = TreeView_GetChild(GetDlgItem(hWnd, ID_TREE), root_item);
		while (hItem != NULL) {
			delete_item = hItem;
			hItem = TreeView_GetNextSibling(GetDlgItem(hWnd, ID_TREE), hItem);
			if ((hash = treeview_get_node_name(GetDlgItem(hWnd, ID_TREE), delete_item, name)) != 0) {
				for (i = 0; i < nmi_count; i++) {
					if (hash == nmi[i].name_hash && lstrcmpi(name, nmi[i].name) == 0) {
						break;
					}
				}
				if (i >= nmi_count) {
					treeview_free_node_info(GetDlgItem(hWnd, ID_TREE), delete_item);
					TreeView_DeleteItem(GetDlgItem(hWnd, ID_TREE), delete_item);
				} 
			}
		}
		break;
	case EXTRACT_TYPE_DATE:
		treeview_sync_date(GetDlgItem(hWnd, ID_TREE), root_item);
		break;
	}
}

/*
 * set_treeview_node - c[̃m[hݒ
 */
static void set_treeview_node(const HWND hWnd, HTREEITEM root_item)
{
	BOOL expand = FALSE;
	int i;

	if (root_item == NULL) {
		root_item = TreeView_GetRoot(GetDlgItem(hWnd, ID_TREE));
	}
	if (TreeView_GetChild(GetDlgItem(hWnd, ID_TREE), root_item) == NULL) {
		expand = TRUE;
	}
	switch (op.extract_type) {
	case EXTRACT_TYPE_NAME:
		for (i = 0; i < nmi_count; i++) {
			treeview_add_node_name(GetDlgItem(hWnd, ID_TREE), root_item, nmi[i].name);
		}
		break;
	case EXTRACT_TYPE_DATE:
		for (i = 0; i < dti_count; i++) {
			treeview_add_node_date(GetDlgItem(hWnd, ID_TREE), root_item, &dti[i]);
		}
		break;
	case EXTRACT_TYPE_ALL:
	default:
		treeview_add_node_all(GetDlgItem(hWnd, ID_TREE), root_item);
		break;
	}
	if (expand == TRUE) {
		TreeView_Expand(GetDlgItem(hWnd, ID_TREE), root_item, TVE_EXPAND);
	}
}

/*
 * set_treeview_root - c[̃[gm[hݒ
 */
static void set_treeview_root(const HWND hWnd)
{
	HCURSOR old_cursor;
	HTREEITEM hItem;
	TCHAR *title;

	old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	SendMessage(GetDlgItem(hWnd, ID_TREE), WM_SETREDRAW, (WPARAM)FALSE, 0);
	// c[r[̃m[h폜
	treeview_free_node_info(GetDlgItem(hWnd, ID_TREE), TreeView_GetRoot(GetDlgItem(hWnd, ID_TREE)));
	TreeView_DeleteAllItems(GetDlgItem(hWnd, ID_TREE));
	// c[r[Ƀm[hݒ
	switch (op.extract_type) {
	case EXTRACT_TYPE_NAME:
		title = message_get_res(IDS_STRING_TYPE_NAME);
		break;
	case EXTRACT_TYPE_DATE:
		title = message_get_res(IDS_STRING_TYPE_DATE);
		break;
	case EXTRACT_TYPE_ALL:
	default:
		title = message_get_res(IDS_STRING_TYPE_ALL);
		break;
	}
	hItem = treeview_set_item(GetDlgItem(hWnd, ID_TREE), title, (HTREEITEM)TVI_ROOT, (HTREEITEM)TVI_LAST, ICON_ROOT, ICON_ROOT, (LPARAM)0);
	// m[hݒ
	set_treeview_node(hWnd, hItem);
	TreeView_SelectItem(GetDlgItem(hWnd, ID_TREE), hItem);
	SendMessage(GetDlgItem(hWnd, ID_TREE), WM_SETREDRAW, (WPARAM)TRUE, 0);
	UpdateWindow(GetDlgItem(hWnd, ID_LIST));
	SetCursor(old_cursor);
}

/*
 * set_toolbar - c[o[̐ݒ
 */
static void set_toolbar(const HWND hWnd, UINT id, int check)
{
	SendMessage(GetDlgItem(hWnd, ID_TOOLBAR), TB_CHECKBUTTON, id, (LPARAM) MAKELONG(check, 0));
	SendMessage(GetDlgItem(hWnd, ID_TOOLBAR), TB_PRESSBUTTON, id, (LPARAM) MAKELONG(check, 0));
}

/*
 * set_statusbar_count - Xe[^Xo[̐ݒ
 */
static void set_statusbar_count(const HWND hWnd)
{
	TCHAR buf[BUF_SIZE];

	if ((exi.filter_name_enable != 0 && *exi.filter_name != TEXT('\0')) ||
		(exi.filter_vs_name_enable != 0 && *exi.filter_vs_name != TEXT('\0')) ||
		exi.filter_date_enable != 0 || exi.filter_start_score_enable != 0) {
		wsprintf(buf, message_get_res(IDS_STRING_STB_DATA_F), ListView_GetItemCount(GetDlgItem(hWnd, ID_LIST)));
	} else {
		wsprintf(buf, message_get_res(IDS_STRING_STB_DATA), ListView_GetItemCount(GetDlgItem(hWnd, ID_LIST)));
	}
	statusbar_set_text(GetDlgItem(hWnd, ID_STATUSBAR), buf, 0);
}

/*
 * set_statusbar_info - Xe[^Xo[̐ݒ
 */
static void set_statusbar_info(const HWND hWnd, const TCHAR *buf)
{
	statusbar_set_text(GetDlgItem(hWnd, ID_STATUSBAR), buf, 1);
}

/*
 * set_controls - Rg[̐ݒ
 */
static void set_controls(const HWND hWnd)
{
	RECT window_rect;
	RECT toolbar_rect;
	DWORD toolbar_size = 0;
	RECT statusbar_rect;
	DWORD statusbar_size = 0;

	GetClientRect(hWnd, (LPRECT)&window_rect);

	// ToolBar̃TCY̎擾
	if (IsWindowVisible(GetDlgItem(hWnd, ID_TOOLBAR)) != 0) {
		GetWindowRect(GetDlgItem(hWnd, ID_TOOLBAR), (LPRECT)&toolbar_rect);
		toolbar_size = (toolbar_rect.bottom - toolbar_rect.top);
	}
	// StatusBar̃TCY̎擾
	if (IsWindowVisible(GetDlgItem(hWnd, ID_STATUSBAR)) != 0) {
		GetWindowRect(GetDlgItem(hWnd, ID_STATUSBAR), (LPRECT)&statusbar_rect);
		statusbar_size = (statusbar_rect.bottom - statusbar_rect.top);
	}
	
	// TreeVieẅʒuATCY̐ݒ
	MoveWindow(GetDlgItem(hWnd, ID_TREE),
		0, toolbar_size, op.sep_size, window_rect.bottom - statusbar_size - toolbar_size, TRUE);
	UpdateWindow(GetDlgItem(hWnd, ID_TREE));

	//Xgr[̃TCYݒ
	MoveWindow(GetDlgItem(hWnd, ID_LIST),
		op.sep_size + (FRAME_CNT * 2), toolbar_size,
		window_rect.right - op.sep_size - (FRAME_CNT * 2), window_rect.bottom - statusbar_size - toolbar_size, TRUE);
	UpdateWindow(GetDlgItem(hWnd, ID_LIST));
}

/*
 * set_menu - j[ݒ
 */
static void set_menu(const HWND hWnd)
{
	UINT enable;
	int i;

	// \j[̐ݒ
	set_toolbar(hWnd, ID_MENUITEM_VIEW_NAME, 0);
	set_toolbar(hWnd, ID_MENUITEM_VIEW_DATE, 0);
	set_toolbar(hWnd, ID_MENUITEM_VIEW_ALL, 0);

	switch (op.extract_type) {
	case EXTRACT_TYPE_NAME:
		CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), MENU_POS_VIEW), ID_MENUITEM_VIEW_NAME,
			ID_MENUITEM_VIEW_ALL, ID_MENUITEM_VIEW_NAME, MF_BYCOMMAND);
		set_toolbar(hWnd, ID_MENUITEM_VIEW_NAME, 1);
		break;
	case EXTRACT_TYPE_DATE:
		CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), MENU_POS_VIEW), ID_MENUITEM_VIEW_NAME,
			ID_MENUITEM_VIEW_ALL, ID_MENUITEM_VIEW_DATE, MF_BYCOMMAND);
		set_toolbar(hWnd, ID_MENUITEM_VIEW_DATE, 1);
		break;
	case EXTRACT_TYPE_ALL:
		CheckMenuRadioItem(GetSubMenu(GetMenu(hWnd), MENU_POS_VIEW), ID_MENUITEM_VIEW_NAME,
			ID_MENUITEM_VIEW_ALL, ID_MENUITEM_VIEW_ALL, MF_BYCOMMAND);
		set_toolbar(hWnd, ID_MENUITEM_VIEW_ALL, 1);
		break;
	}
	// f[^j[̐ݒ
	if ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), -1, LVNI_SELECTED)) != -1 &&
		g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)].index >= 0) {
		enable = MF_ENABLED;
	} else {
		enable = MF_GRAYED;
	}
	EnableMenuItem(GetSubMenu(GetMenu(hWnd), MENU_POS_DATA), ID_MENUITEM_DATA_OUT, enable);

	if ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), -1, LVNI_SELECTED)) != -1) {
		enable = MF_ENABLED;
	} else {
		enable = MF_GRAYED;
	}
	EnableMenuItem(GetSubMenu(GetMenu(hWnd), MENU_POS_DATA), ID_MENUITEM_DATA_VIEW, enable);
	EnableMenuItem(GetSubMenu(GetMenu(hWnd), MENU_POS_DATA), ID_MENUITEM_DATA_COPY, enable);
	EnableMenuItem(GetSubMenu(GetMenu(hWnd), MENU_POS_DATA), ID_MENUITEM_DATA_SPLIT, enable);
	EnableMenuItem(GetSubMenu(GetMenu(hWnd), MENU_POS_DATA), ID_MENUITEM_DATA_DELETE, enable);

	if (IsClipboardFormatAvailable(CF_HDROP) == 0) {
		enable = MF_GRAYED;
	} else {
		enable = MF_ENABLED;
	}
	EnableMenuItem(GetSubMenu(GetMenu(hWnd), MENU_POS_DATA), ID_MENUITEM_DATA_PASTE, enable);
}

/*
 * set_window_title - EBhE^Cgݒ
 */
static void set_window_title(const HWND hWnd)
{
	TCHAR buf[MAX_PATH + 15];

	wsprintf(buf, TEXT("%s - [%s]"), WINDOW_TITLE, csv_path);
	SetWindowText(hWnd, buf);
}

/*
 * main_proc - CEBhEvV[W
 */
static LRESULT CALLBACK main_proc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static HWND focus_wnd;
	static BOOL sep_move;
	HCURSOR old_cursor;
	HIMAGELIST icon_list;
	DATA_INFO *di;
	TCHAR err_str[BUF_SIZE];
	int i, j;
	int st, en;
	BOOL update = FALSE;

	switch (msg) {
	case WM_CREATE:
		GetWindowRect(hWnd, (LPRECT)&op.window_rect);
		op.window_rect.right -= op.window_rect.left;
		op.window_rect.bottom -= op.window_rect.top;
		// T[oJn
		server_start(hWnd);
		// c[o[
		toolbar_create(hInst, hWnd, ID_TOOLBAR);
		// C[WXg
		icon_list = create_imagelist(hInst);
		// c[r[
		treeview_create(hInst, hWnd, ID_TREE, icon_list);
		// Xgr[
		listview_create(hInst, hWnd, ID_LIST, icon_list);
		listview_set_column(GetDlgItem(hWnd, ID_LIST), op.hi, op.hi_count);
		// Xe[^Xo[
		statusbar_create(hWnd, ID_STATUSBAR);
		// ʒuATCYݒ
		set_controls(hWnd);
		focus_wnd = GetDlgItem(hWnd, ID_TREE);
		SetFocus(focus_wnd);
		set_treeview_root(hWnd);
		set_menu(hWnd);
		if (op.csv_path_default != 0) {
			wsprintf(csv_path, TEXT("%s\\%s"), work_path, CSV_PATH);
			create_tree_directory(csv_path);
		} else if (*op.csv_path == TEXT('\0')) {
			SetTimer(hWnd, ID_TIMER_INIT, 1, NULL);
			break;
		} else {
			lstrcpy(csv_path, op.csv_path);
		}
		set_window_title(hWnd);
		if (index_load(hWnd, index_file, &g_di, &g_di_count, &g_di_size) == FALSE) {
			DeleteFile(index_file);
		}
		filelist_create(g_di, g_di_count);
		treeinfo_create_list(g_di, 0, g_di_count);
		set_treeview_root(hWnd);
		PostMessage(hWnd, WM_MAIN_RELOAD_ALL, 0, 0);
		break;

	case WM_SETFOCUS:
		SetFocus(focus_wnd);
		break;

	case WM_SIZE:
		op.window_state = (IsZoomed(hWnd) == 0) ? SW_SHOWDEFAULT : SW_MAXIMIZE;
		SendMessage(GetDlgItem(hWnd, ID_TOOLBAR), WM_SIZE, wParam, lParam);
		SendMessage(GetDlgItem(hWnd, ID_STATUSBAR), WM_SIZE, wParam, lParam);
		set_controls(hWnd);
		break;
	
	case WM_EXITSIZEMOVE:
		// TCYύX
		if (IsWindowVisible(hWnd) != 0 && IsIconic(hWnd) == 0 && IsZoomed(hWnd) == 0) {
			GetWindowRect(hWnd, (LPRECT)&op.window_rect);
			op.window_rect.right -= op.window_rect.left;
			op.window_rect.bottom -= op.window_rect.top;
		}
		break;

	case WM_QUERYENDSESSION:
		return TRUE;
	
	case WM_ENDSESSION:
		treeview_free_node_info(GetDlgItem(hWnd, ID_TREE), TreeView_GetRoot(GetDlgItem(hWnd, ID_TREE)));
		TreeView_DeleteAllItems(GetDlgItem(hWnd, ID_TREE));
		// C[WXg̉
		ImageList_Destroy((void *)TreeView_SetImageList(GetDlgItem(hWnd, ID_TREE), NULL, TVSIL_NORMAL));
		treeview_close(GetDlgItem(hWnd, ID_TREE));
		listview_close(GetDlgItem(hWnd, ID_LIST));
		DestroyWindow(hWnd);
		return 0;
	
	case WM_CLOSE:
		treeview_free_node_info(GetDlgItem(hWnd, ID_TREE), TreeView_GetRoot(GetDlgItem(hWnd, ID_TREE)));
		TreeView_DeleteAllItems(GetDlgItem(hWnd, ID_TREE));
		// C[WXg̉
		ImageList_Destroy((void *)TreeView_SetImageList(GetDlgItem(hWnd, ID_TREE), NULL, TVSIL_NORMAL));
		treeview_close(GetDlgItem(hWnd, ID_TREE));
		listview_close(GetDlgItem(hWnd, ID_LIST));
		DestroyWindow(hWnd);
		break;
	
	case WM_DESTROY:
		// ݒۑ
		ini_put_option(ini_file);
		// f[^
		mem_free(&g_show_di);
		g_show_di_count = 0;
		g_show_di_size = 0;
		mem_free(&g_di);
		g_di_count = 0;
		g_di_size = 0;
		g_file_all = 0;
		treeinfo_free();
		filelist_free();
		PostQuitMessage(0);
		break;
	
	case WM_DROPFILES:
		// t@C̃hbv
		SetForegroundWindow(hWnd);
		hdrop_to_file(hWnd, (HDROP)wParam);
		break;

	case WM_LBUTTONDOWN:
		// Ëړ
		if (GetForegroundWindow() != hWnd) {
			break;
		}
		if (sep_move == FALSE) {
			if (frame_initialize(hWnd) == FALSE) {
				break;
			}
			sep_move = TRUE;
			frame_draw(hWnd, GetDlgItem(hWnd, ID_LIST));
			SetTimer(hWnd, ID_TIMER_SEP, 1, NULL);
		}
		break;

	case WM_MOUSEMOVE:
		if (sep_move == TRUE) {
			frame_draw(hWnd, GetDlgItem(hWnd, ID_LIST));
		}
		break;

	case WM_LBUTTONUP:
		if (sep_move == TRUE) {
			int ret;
			KillTimer(hWnd, ID_TIMER_SEP);
			sep_move = FALSE;
			if ((ret = frame_draw_end(hWnd)) == -1) {
				break;
			}
			op.sep_size = ret;
			set_controls(hWnd);
		}
		break;

	case WM_TIMER:
		switch (wParam) {
		case ID_TIMER_SEP:
			// Ëړ
			if (hWnd != GetForegroundWindow() || GetAsyncKeyState(VK_ESCAPE) < 0 ||
				GetAsyncKeyState(VK_RBUTTON) < 0) {
				KillTimer(hWnd, wParam);
				frame_free();
				sep_move = FALSE;
				InvalidateRect(hWnd, NULL, FALSE);
				UpdateWindow(hWnd);
			}
			break;

		case ID_TIMER_INIT:
			KillTimer(hWnd, ID_TIMER_INIT);
			// tH_ݒ
			SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_FILE_SET_FOLDER, 0);
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_KEY_TAB:
			// tH[JX؂ւ
			focus_wnd = GetDlgItem(hWnd, (GetFocus() == GetDlgItem(hWnd, ID_LIST)) ? ID_TREE : ID_LIST);
			SetFocus(focus_wnd);
			break;

		case ID_KEY_ESC:
			// t@C̃LZ
			if (g_file_all == 0) {
				break;
			}
			csv_read_cancel(TRUE);
			break;

		case ID_MENUITEM_FILE_SET_FOLDER:
			// f[^tH_ݒ
			if (g_file_all != 0) {
				break;
			}
			if (select_folder(hWnd, message_get_res(IDS_STRING_MSG_FOLDER), op.csv_path) == TRUE || *op.csv_path == TEXT('\0')) {
				if (*op.csv_path == TEXT('\0')) {
					op.csv_path_default = 1;
					wsprintf(csv_path, TEXT("%s\\%s"), work_path, CSV_PATH);
					create_tree_directory(csv_path);
				} else {
					op.csv_path_default = 0;
					lstrcpy(csv_path, op.csv_path);
				}
				// ݒۑ
				ini_put_option(ini_file);
				set_window_title(hWnd);
				treeview_free_node_info(GetDlgItem(hWnd, ID_TREE), TreeView_GetRoot(GetDlgItem(hWnd, ID_TREE)));
				TreeView_DeleteAllItems(GetDlgItem(hWnd, ID_TREE));
				mem_free(&g_di);
				g_di_count = 0;
				g_di_size = 0;
				g_file_all = 0;
				DeleteFile(index_file);
				treeinfo_free();
				filelist_create(g_di, g_di_count);
				treeinfo_create_list(g_di, 0, g_di_count);
				set_treeview_root(hWnd);
				SendMessage(hWnd, WM_MAIN_RELOAD_ALL, 0, 0);
			}
			break;

		case ID_MENUITEM_FILE_RELOAD:
			if (g_file_all != 0) {
				break;
			}
			treeview_free_node_info(GetDlgItem(hWnd, ID_TREE), TreeView_GetRoot(GetDlgItem(hWnd, ID_TREE)));
			TreeView_DeleteAllItems(GetDlgItem(hWnd, ID_TREE));
			mem_free(&g_di);
			g_di_count = 0;
			g_di_size = 0;
			g_file_all = 0;
			DeleteFile(index_file);
			treeinfo_free();
			filelist_create(g_di, g_di_count);
			treeinfo_create_list(g_di, 0, g_di_count);
			set_treeview_root(hWnd);
			SendMessage(hWnd, WM_MAIN_RELOAD_ALL, 0, 0);
			break;

		case ID_MENUITEM_FILE_OUT:
			data_show_list(hWnd, GetDlgItem(hWnd, ID_LIST), g_show_di, g_show_di_count);
			break;

		case ID_MENUITEM_FILE_EXIT:
			// I
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			break;

		case ID_MENUITEM_VIEW_NAME:
			// vC[ɕ\
			op.extract_type = EXTRACT_TYPE_NAME;
			set_treeview_root(hWnd);
			set_menu(hWnd);
			break;

		case ID_MENUITEM_VIEW_DATE:
			// tɕ\
			op.extract_type = EXTRACT_TYPE_DATE;
			set_treeview_root(hWnd);
			set_menu(hWnd);
			break;

		case ID_MENUITEM_VIEW_ALL:
			// Sf[^
			op.extract_type = EXTRACT_TYPE_ALL;
			set_treeview_root(hWnd);
			set_menu(hWnd);
			break;

		case ID_MENUITEM_VIEW_FILTER:
			// tB^[ݒ
			if (show_filter(hInst, hWnd, &exi) == TRUE) {
				SendMessage(hWnd, WM_TV_EVENT, TVN_SELCHANGED, 0);
			}
			break;

		case ID_MENUITEM_VIEW_COLUMN:
			// ̑I
			if (show_column(hInst, hWnd) == TRUE) {
				old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
				SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)FALSE, 0);
				listview_set_column(GetDlgItem(hWnd, ID_LIST), op.hi, op.hi_count);
				SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)TRUE, 0);
				InvalidateRect(GetDlgItem(hWnd, ID_LIST), NULL, FALSE);
				UpdateWindow(GetDlgItem(hWnd, ID_LIST));
				// ݒۑ
				ini_put_option(ini_file);
				SetCursor(old_cursor);
			}
			break;

		case ID_MENUITEM_VIEW_OPTION:
			// IvV
			if (show_option(hInst, hWnd) == TRUE) {
				// ݒۑ
				ini_put_option(ini_file);
			}
			break;

		case ID_MENUITEM_VIEW_REFRESH:
			// ŐV̏ɍXV
			treeinfo_free();
			treeinfo_create_list(g_di, 0, g_di_count);
			sync_treeview_node(hWnd, NULL);
			set_treeview_node(hWnd, NULL);
			SendMessage(hWnd, WM_TV_EVENT, TVN_SELCHANGED, 0);
			SendMessage(hWnd, WM_MAIN_RELOAD_ALL, 0, 0);
			break;

		case ID_MENUITEM_DATA_VIEW:
			// \
			if ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), -1, LVNI_SELECTED | LVNI_FOCUSED)) == -1) {
				break;
			}
			SetCurrentDirectory(work_path);
			if (g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)].index < 0) {
				show_view(hInst, hWnd, listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i));
			} else {
				show_player(hInst, hWnd, g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)].index);
			}
			break;

		case ID_MENUITEM_DATA_OUT:
			// f[^o
			if ((i = ListView_GetNextItem(GetDlgItem(hWnd, ID_LIST), -1, LVNI_SELECTED | LVNI_FOCUSED)) == -1) {
				break;
			}
			SetCurrentDirectory(work_path);
			data_show_data(hWnd, &g_show_di[listview_get_lparam(GetDlgItem(hWnd, ID_LIST), i)]);
			break;

		case ID_MENUITEM_DATA_SPLIT:
			// f[^
			data_split(hWnd);
			break;

		case ID_MENUITEM_DATA_COPY:
			// Rs[
			SetCurrentDirectory(work_path);
			data_to_clipboard(hWnd);
			break;

		case ID_MENUITEM_DATA_PASTE:
			// \t
			SetCurrentDirectory(work_path);
			if (IsClipboardFormatAvailable(CF_HDROP) == 0) {
				break;
			}
			if (OpenClipboard(hWnd) == FALSE) {
				break;
			}
			hdrop_to_file(hWnd, GetClipboardData(CF_HDROP));
			CloseClipboard();
			break;

		case ID_MENUITEM_DATA_DELETE:
			// 폜
			SetCurrentDirectory(work_path);
			data_delete(hWnd);
			set_statusbar_count(hWnd);
			break;

		case ID_MENUITEM_DATA_SELECT_ALL:
			// ׂđI
			SetFocus(GetDlgItem(hWnd, ID_LIST));
			ListView_SetItemState(GetDlgItem(hWnd, ID_LIST), -1, LVIS_SELECTED, LVIS_SELECTED);
			break;

		case ID_MENUITEM_HELP_ABOUT:
			// o[W
			MessageBox(hWnd,
				APP_NAME
				TEXT("\nCopyright (C) 1996-2006 by Nakashima Tomoaki. All rights reserved.\n\n")
				TEXT("WEB SITE: http://www.nakka.com/\nE-MAIL: nakka@nakka.com"),
				TEXT("About"), MB_OK | MB_ICONINFORMATION);
			break;
		}
		break;

	case WM_INITMENU:
		set_menu(hWnd);
		break;

	case WM_NOTIFY:
		// c[r[
		if (((NMHDR *)lParam)->hwndFrom == GetDlgItem(hWnd, ID_TREE)) {
			return treeview_notify_proc(hWnd, lParam);
		}
		// Xgr[
		if (((NMHDR *)lParam)->hwndFrom == ListView_GetHeader(GetDlgItem(hWnd, ID_LIST))) {
			return listview_header_notify_proc(hWnd, GetDlgItem(hWnd, ID_LIST), lParam);
		}
		if (((NMHDR *)lParam)->hwndFrom == GetDlgItem(hWnd, ID_LIST)) {
			return listview_notify_proc(hWnd, GetDlgItem(hWnd, ID_LIST), lParam);
		}
		// c[o[
		if (((NMHDR *)lParam)->code == TTN_NEEDTEXT) {
			((TOOLTIPTEXT*)lParam)->hinst = hInst;
			((TOOLTIPTEXT*)lParam)->lpszText = MAKEINTRESOURCE(((NMHDR *)lParam)->idFrom);
		}
		break;

	case WM_TV_EVENT:
		// c[r[Cxg
		switch (wParam) {
		case TVN_SELCHANGED:
			old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
			if (treeview_get_node_info(GetDlgItem(hWnd, ID_TREE),
				TreeView_GetSelection(GetDlgItem(hWnd, ID_TREE)), &exi) == FALSE) {
				SetCursor(old_cursor);
				return TRUE;
			}
			// \ACe̒o
			mem_free(&g_show_di);
			g_show_di_count = 0;
			g_show_di_size = g_di_count + DATA_ALLOC_SIZE;
			g_show_di = (DATA_INFO *)mem_calloc(sizeof(DATA_INFO) * g_show_di_size);
			if (g_show_di == NULL) {
				g_show_di_size = 0;
				SetCursor(old_cursor);
				message_get_error(GetLastError(), err_str);
				MessageBox(hWnd, err_str, WINDOW_TITLE, MB_ICONERROR);
				break;
			}
			data_create_stats(g_di, 0, g_di_count, g_show_di, &g_show_di_count, FALSE);
			// Xgr[ɃACeݒ
			SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)FALSE, 0);
			ListView_DeleteAllItems(GetDlgItem(hWnd, ID_LIST));
			ListView_SetItemCount(GetDlgItem(hWnd, ID_LIST), g_show_di_count);
			for (i = 0; i < g_show_di_count; i++) {
				listview_set_item(GetDlgItem(hWnd, ID_LIST), i, i);
			}
			listview_sort(GetDlgItem(hWnd, ID_LIST), op.sort_colum, op.sort_order);
			ListView_SetItemState(GetDlgItem(hWnd, ID_LIST), 0, LVIS_FOCUSED, LVIS_FOCUSED);
			SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)TRUE, 0);
			UpdateWindow(GetDlgItem(hWnd, ID_LIST));
			set_statusbar_count(hWnd);
			SetCursor(old_cursor);
			return TRUE;

		case NM_SETFOCUS:
			focus_wnd = GetDlgItem(hWnd, ID_TREE);
			break;
		}
		break;

	case WM_LV_EVENT:
		switch (wParam) {
		case LVN_GETDISPINFO:
			listview_get_disp_item(GetDlgItem(hWnd, ID_LIST), (NMLVDISPINFO *)lParam);
			break;

		case LVN_ITEMACTIVATE:
			SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_DATA_VIEW, 0);
			break;

		case LVN_KEYDOWN:
			switch (((LV_KEYDOWN *)lParam)->wVKey) {
			case 'A':
				if (GetKeyState(VK_CONTROL) < 0) {
					// ׂđI
					SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_DATA_SELECT_ALL, 0);
					return TRUE;
				}
				break;

			case VK_DELETE:
				SendMessage(hWnd, WM_COMMAND, ID_MENUITEM_DATA_DELETE, 0);
				return TRUE;

			case VK_APPS:
				SendMessage(hWnd, WM_LV_EVENT, NM_RCLICK, 0);
				return TRUE;

			case VK_BACK:
				if (TreeView_GetParent(GetDlgItem(hWnd, ID_TREE),
					TreeView_GetSelection(GetDlgItem(hWnd, ID_TREE))) != NULL) {
					TreeView_SelectItem(GetDlgItem(hWnd, ID_TREE),
						TreeView_GetParent(GetDlgItem(hWnd, ID_TREE),
						TreeView_GetSelection(GetDlgItem(hWnd, ID_TREE))));
				}
				return TRUE;
			}
			break;

		case NM_RCLICK:
			{
				POINT apos;
				// j[̕\
				GetCursorPos((LPPOINT)&apos);
				TrackPopupMenu(GetSubMenu(GetMenu(hWnd), MENU_POS_DATA), TPM_TOPALIGN, apos.x, apos.y, 0, hWnd, NULL);
			}
			break;

		case NM_SETFOCUS:
			focus_wnd = GetDlgItem(hWnd, ID_LIST);
			break;
		}
		break;

	case WM_MAIN_REFRESH:
		// ʕ\XV
		treeinfo_free();
		treeinfo_create_list(g_di, 0, g_di_count);
		set_treeview_node(hWnd, NULL);
		SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)TRUE, 0);
		InvalidateRect(GetDlgItem(hWnd, ID_LIST), NULL, FALSE);
		UpdateWindow(GetDlgItem(hWnd, ID_LIST));
		set_statusbar_info(hWnd, TEXT(""));
		set_statusbar_count(hWnd);
		break;

	case WM_MAIN_RELOAD_FILE:
		// f[^XV
		if ((TCHAR *)lParam == NULL) {
			break;
		}
		SetCurrentDirectory(work_path);
		set_statusbar_info(hWnd, message_get_res(IDS_STRING_STB_RELOAD_DATA));
		if ((j = csv_find_index((TCHAR *)lParam, g_di, g_di_count)) != -1) {
			// ɓo^ς݂̃f[^XV
			data_subtract_stats(g_di, j, j + 2, g_show_di, &g_show_di_count);
			if (csv_read_file(&g_di[j], csv_path, (TCHAR *)lParam, j) == -1) {
				set_statusbar_info(hWnd, TEXT(""));
				break;
			}
			SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)FALSE, 0);
			if (op.auto_split == 1) {
				// 
				data_split_data(hWnd, op.auto_split_sep, op.auto_split_delete, &g_di, &g_di_count, &g_di_size, j);
				data_split_data(hWnd, op.auto_split_sep, op.auto_split_delete, &g_di, &g_di_count, &g_di_size, j + 1);
			}
			// ACe\XV
			SendMessage(hWnd, WM_MAIN_UPDATE_ITEM, j, j + 2);
			if (wParam == 1) {
				break;
			}
			// c[r[̍XV
			treeinfo_create_list(g_di, j, j + 2);
		} else {
			// VKǉ
			st = g_di_count;
			if (csv_read(csv_path, (TCHAR *)lParam, &g_di, &g_di_count, &g_di_size) == FALSE) {
				set_statusbar_info(hWnd, TEXT(""));
				break;
			}
			SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)FALSE, 0);
			// ACe\XV
			SendMessage(hWnd, WM_MAIN_RELOAD_ITEM, st, g_di_count);
			if (wParam == 1) {
				break;
			}
			// c[r[̍XV
			treeinfo_create_list(g_di, st, g_di_count);
		}
		set_treeview_node(hWnd, NULL);
		SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)TRUE, 0);
		InvalidateRect(GetDlgItem(hWnd, ID_LIST), NULL, FALSE);
		UpdateWindow(GetDlgItem(hWnd, ID_LIST));
		set_statusbar_info(hWnd, TEXT(""));
		set_statusbar_count(hWnd);
		break;

	case WM_MAIN_RELOAD_ALL:
		// t@C
		if (g_file_all != 0) {
			break;
		}
		SetCurrentDirectory(work_path);
		old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
		set_statusbar_info(hWnd, message_get_res(IDS_STRING_STB_FIND_FILE));
		// t@C̏
		g_st = g_di_count;
		g_file_all = csv_count(csv_path);
		if ((g_file_all * 2) > g_di_size) {
			// Ċm
			di = (DATA_INFO *)mem_calloc(sizeof(DATA_INFO) * ((g_file_all * 2) + DATA_ALLOC_SIZE));
			if (di == NULL) {
				message_get_error(GetLastError(), err_str);
				MessageBox(hWnd, err_str, WINDOW_TITLE, MB_ICONERROR);
				set_statusbar_info(hWnd, TEXT(""));
				SetCursor(old_cursor);
				g_file_all = 0;
				break;
			}
			if (g_di != NULL) {
				CopyMemory(di, g_di, sizeof(DATA_INFO) * g_di_count);
				mem_free(&g_di);
			}
			g_di = di;
			g_di_size = g_file_all * 2 + DATA_ALLOC_SIZE;
		}
		filelist_alloc(g_file_all);
		SetCursor(old_cursor);
		// t@Cǂݍ
		j = FILE_READ_COUNT;
		csv_read_cancel(FALSE);
		csv_read_sync(hWnd, csv_path, &g_di, &g_di_count, &g_di_size, &j);
		if (IsWindow(hWnd) == FALSE) {
			ExitProcess(0);
			break;
		}
		g_file_all = 0;
		SendMessage(hWnd, WM_MAIN_RELOAD_STATUS, 0, 0);
		set_statusbar_info(hWnd, TEXT(""));
		break;

	case WM_MAIN_RELOAD_STATUS:
		// t@C̕\XV
		if (g_file_all > 0) {
			TCHAR buf[BUF_SIZE];
			i = (int)(((float)filelist_get_count() / (float)g_file_all) * 100);
			if (i > 100) i = 100;
			wsprintf(buf, TEXT("%s %d %%"), message_get_res(IDS_STRING_STB_FIND_FILE), i);
			set_statusbar_info(hWnd, buf);
		}
		// c[r[̍XV
		treeinfo_create_list(g_di, g_st, g_di_count);
		set_treeview_node(hWnd, NULL);
		SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)FALSE, 0);
		// ACe\XV
		SendMessage(hWnd, WM_MAIN_RELOAD_ITEM, g_st, g_di_count);
		g_st = g_di_count;
		SendMessage(GetDlgItem(hWnd, ID_LIST), WM_SETREDRAW, (WPARAM)TRUE, 0);
		InvalidateRect(GetDlgItem(hWnd, ID_LIST), NULL, FALSE);
		UpdateWindow(GetDlgItem(hWnd, ID_LIST));
		set_statusbar_count(hWnd);
		break;

	case WM_MAIN_UPDATE_ITEM:
		update = TRUE;
	case WM_MAIN_RELOAD_ITEM:
		st = wParam;
		en = lParam;
		if (st >= en) {
			break;
		}
		// CfbNX̕ۑ
		if (index_save(index_file, g_di, st, en) == FALSE && update == FALSE) {
			message_get_error(GetLastError(), err_str);
			MessageBox(hWnd, err_str, WINDOW_TITLE, MB_ICONERROR);
		}
		if (g_show_di_count == 0) {
			SendMessage(hWnd, WM_TV_EVENT, TVN_SELCHANGED, 0);
			break;
		}
		if (g_show_di_count + (en - st) > g_show_di_size) {
			// Ċm
			di = (DATA_INFO *)mem_calloc(sizeof(DATA_INFO) * (g_show_di_count + (en - st) + DATA_ALLOC_SIZE));
			if (di == NULL) {
				message_get_error(GetLastError(), err_str);
				MessageBox(hWnd, err_str, WINDOW_TITLE, MB_ICONERROR);
				break;
			}
			if (g_show_di != NULL) {
				CopyMemory(di, g_show_di, sizeof(DATA_INFO) * g_show_di_count);
				mem_free(&g_show_di);
			}
			g_show_di = di;
			g_show_di_size = g_show_di_count + (en - st) + DATA_ALLOC_SIZE;
		}
		// \ACe̒o
		j = g_show_di_count;
		data_create_stats(g_di, st, en, g_show_di, &g_show_di_count, update);
		// Xgr[ɃACeݒ
		for (i = j; i < g_show_di_count; i++) {
			listview_set_item(GetDlgItem(hWnd, ID_LIST), i, i);
		}
		break;

	case WM_MAIN_GET_CSV_PATH:
		if (lParam == 0) {
			break;
		}
		SetCurrentDirectory(work_path);
		lstrcpy((TCHAR *)lParam, csv_path);
		break;

	case WM_MAIN_SOCK_ERROR:
		if (lParam == 0) {
			break;
		}
		wsprintf(err_str, message_get_res(IDS_STRING_MSG_SOCK_ERROR), (TCHAR *)lParam);
		MessageBox(hWnd, err_str, WINDOW_TITLE, MB_ICONERROR);
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * get_path - pX̍쐬
 */
static void get_path(const HINSTANCE hInstance)
{
	TCHAR *p, *r;

	// AvP[ṼpX擾
	GetModuleFileName(hInstance, work_path, MAX_PATH - 1);
	for (p = r = work_path; *p != TEXT('\0'); p++) {
#ifndef UNICODE
		if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
			p++;
			continue;
		}
#endif	// UNICODE
		if (*p == TEXT('\\') || *p == TEXT('/')) {
			r = p;
		}
	}
	*r = TEXT('\0');
	// init@CpX쐬
	wsprintf(ini_file, TEXT("%s\\%s"), work_path, INI_FILE);
	// CfbNXt@CpX쐬
	wsprintf(index_file, TEXT("%s\\%s"), work_path, INDEX_FILE);
	// V[go͗ppX쐬
	wsprintf(out_data_path, TEXT("%s\\%s"), work_path, OUT_DATA_PATH);
	wsprintf(out_list_path, TEXT("%s\\%s"), work_path, OUT_LIST_PATH);
}

/*
 * init_application - EBhENX̓o^
 */
static BOOL init_application(HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.hCursor = LoadCursor(NULL, IDC_SIZEWE);
	wc.lpszMenuName = MAKEINTRESOURCE(IDR_MENU_MAIN);
	wc.lpfnWndProc = (WNDPROC)main_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_MAIN));
	wc.hbrBackground = (HBRUSH)COLOR_BTNSHADOW;
	wc.lpszClassName = MAIN_WND_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * init_instance - EBhE̍쐬
 */
static HWND init_instance(HINSTANCE hInstance, int CmdShow)
{
	HWND hWnd =  NULL;

	// EBhE̍쐬
	hWnd = CreateWindowEx(WS_EX_ACCEPTFILES,
		MAIN_WND_CLASS,
		WINDOW_TITLE,
		WS_OVERLAPPEDWINDOW,
		op.window_rect.left,
		op.window_rect.top,
		op.window_rect.right,
		op.window_rect.bottom,
		NULL, NULL, hInstance, NULL);
	if (hWnd == NULL) {
		return NULL;
	}

	// EBhE̕\
	ShowWindow(hWnd, CmdShow);
	UpdateWindow(hWnd);
	return hWnd;
}

/*
 * do_events - Cxg
 */
void do_events(const HWND hWnd)
{
	MSG msg;

	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) == TRUE) {
		if (TranslateAccelerator(hWnd, hAccel, &msg) == TRUE) {
			continue;
		}
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

/*
 * WinMain - C
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
#ifndef _DEBUG
	HANDLE hMutex;
#endif
	HINSTANCE hLib;
	FARPROC _InitCommonControlsEx;
	INITCOMMONCONTROLSEX icx;
	HWND hWnd;
	MSG msg;
	TCHAR err_str[BUF_SIZE];

	hInst = hInstance;
	
#ifndef _DEBUG
	// 2dN`FbN
	if ((hMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, MUTEX)) == NULL) {
		hMutex = CreateMutex(NULL, TRUE, MUTEX);
	} else {
		SetForegroundWindow(FindWindow(MAIN_WND_CLASS, NULL));
		CloseHandle(hMutex);
		return 0;
	}
#endif
	// CommonControl̏
	InitCommonControls();
	hLib = LoadLibrary(TEXT("comctl32.dll"));
	if(hLib != NULL){
		_InitCommonControlsEx = GetProcAddress(hLib, "InitCommonControlsEx");
		if(_InitCommonControlsEx != NULL){
			icx.dwSize = sizeof(INITCOMMONCONTROLSEX);
			icx.dwICC = ICC_WIN95_CLASSES | ICC_DATE_CLASSES;
			_InitCommonControlsEx(&icx);
		}
		FreeLibrary(hLib);
	}

	// ݒ擾
	get_path(hInst);
	if (ini_get_option(ini_file) == FALSE) {
		message_get_error(GetLastError(), err_str);
		MessageBox(NULL, err_str, WINDOW_TITLE, MB_ICONERROR);
		return 0;
	}

	// EBhENX̓o^
	if (score_list_regist(hInstance) == FALSE ||
		init_application(hInstance) == FALSE) {
		message_get_error(GetLastError(), err_str);
		MessageBox(NULL, err_str, WINDOW_TITLE, MB_ICONERROR);
		return 0;
	}
	// EBhE̍쐬
	if ((hWnd = init_instance(hInstance, op.window_state)) == NULL) {
		return 0;
	}
	g_main_wnd = hWnd;
	hAccel = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR));
	// EBhEbZ[W
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (TranslateAccelerator(hWnd, hAccel, &msg) == TRUE) {
			continue;
		}
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
#ifdef _DEBUG
	mem_debug();
#endif
	return msg.wParam;
}
/* End of source */
